package com.gx.obe.management.dialog;

import java.io.File;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.concurrent.Callable;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;
import java.util.concurrent.LinkedBlockingQueue;
import java.util.concurrent.ThreadPoolExecutor;
import java.util.concurrent.TimeUnit;

import org.dom4j.Document;
import org.dom4j.DocumentHelper;
import org.dom4j.Element;
import org.eclipse.nebula.widgets.grid.Grid;
import org.eclipse.nebula.widgets.grid.GridColumn;
import org.eclipse.nebula.widgets.grid.GridItem;
import org.eclipse.swt.SWT;
import org.eclipse.swt.core.CallBack;
import org.eclipse.swt.core.MessageDialog;
import org.eclipse.swt.core.utils.GridUtils;
import org.eclipse.swt.core.widgets.EButton;
import org.eclipse.swt.core.widgets.EDialog;
import org.eclipse.swt.core.widgets.ELink;
import org.eclipse.swt.core.widgets.ESepator;
import org.eclipse.swt.events.MouseAdapter;
import org.eclipse.swt.events.MouseEvent;
import org.eclipse.swt.graphics.Point;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Control;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.Label;
import org.eclipse.swt.widgets.Shell;

import com.gx.obe.ColorConstants;
import com.gx.obe.common.file.enumeration.FileConstants;
import com.gx.obe.common.icons.IconsActivator;
import com.gx.obe.common.widget.dialog.ProgressDialog;
import com.gx.obe.components.core.Constants;
import com.gx.obe.components.core.I18NEnum;
import com.gx.obe.message.Messages;
import com.gx.obe.thread.CreateEvaluationStepThread;
import com.gx.obe.util.utils.ObjectUtils;
import com.gx.obe.util.utils.XmlUtils;
import com.gx.obe.web.entity.evaluation.EvaluationStep;
import com.gx.obe.web.entity.tender.TenderProjectEntity;
import com.gx.obe.web.service.TenderProjectService;
import com.gx.utils.XmlBeanUtils;
import com.swtdesigner.ResourceManager;
import com.swtdesigner.SWTResourceManager;

public class ApplyToOtherTenderPackDialog extends EDialog {

	private Grid grid;
	private TenderProjectEntity tenderProjectEntity;
	private List<EvaluationStep> evaluationStepList;
	private TenderProjectService tenderProjectService ;
	private EButton applyToBtn;
	private Label tipsLabel;
	private String tenderPackName = "包件";
	private ThreadPoolExecutor threadPoolExecutor;
	private Map<String, GridItem> tenderPackItemMap = new HashMap<String, GridItem>();

	/**
	 * Create the dialog.
	 * @param parentShell
	 */
	public ApplyToOtherTenderPackDialog(Shell parentShell, TenderProjectEntity tenderProjectEntity, List<EvaluationStep> evaluationStepList) {
		super(parentShell);
		this.tenderProjectEntity = tenderProjectEntity;
		this.evaluationStepList = evaluationStepList;
	}

	/**
	 * Create contents of the dialog.
	 * @param parent
	 */
	@Override
	protected Control createDialogArea(Composite parent) {
		Composite container = (Composite) super.createDialogArea(parent);
		GridLayout gl_container = new GridLayout(1, false);
		gl_container.verticalSpacing = 0;
		gl_container.horizontalSpacing = 0;
		gl_container.marginHeight = 0;
		gl_container.marginWidth = 0;
		container.setLayout(gl_container);
		
		Composite composite = new Composite(container, SWT.NONE);
		GridLayout gl_composite = new GridLayout(1, false);
		gl_composite.verticalSpacing = 0;
		composite.setLayout(gl_composite);
		composite.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		Composite opComposite = new Composite(composite, SWT.NONE);
		opComposite.setLayout(new GridLayout(3, false));
		opComposite.setLayoutData(new GridData(SWT.RIGHT, SWT.FILL, true, false, 1, 1));
		
		ELink checkAllLink = new ELink(opComposite, SWT.NONE);
		checkAllLink.setText(Messages.Button_selectAll);
		checkAllLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/select_all.png"));
		checkAllLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				GridUtils.checkAll(grid, 0, true);
			}
		});
		
		ELink checkBackLink = new ELink(opComposite, SWT.NONE);
		checkBackLink.setText(Messages.Button_selectInvert);
		checkBackLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/switch.png"));
		checkBackLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				GridUtils.checkAllBack(grid, 0);
			}
		});
		
		ELink cancelLink = new ELink(opComposite, SWT.NONE);
		cancelLink.setText(Messages.Button_cancel);
		cancelLink.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/disselect_all.png"));
		cancelLink.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				GridUtils.checkAll(grid, 0, false);
			}
		});
		
		grid = new Grid(composite, SWT.BORDER | SWT.H_SCROLL | SWT.V_SCROLL);
		grid.setHeaderVisible(true);
		grid.setLayoutData(new GridData(SWT.FILL, SWT.FILL, true, true, 1, 1));
		
		GridColumn gridColumn_1 = new GridColumn(grid, SWT.CHECK);
		gridColumn_1.setText("编号");
		gridColumn_1.setWidth(100);
		gridColumn_1.setWordWrap(true);
		
		GridColumn gridColumn_2 = new GridColumn(grid, SWT.NONE);
		gridColumn_2.setText(Messages.Name);
		gridColumn_2.setWidth(150);
		gridColumn_2.setWordWrap(true);
		
		
		GridColumn gridColumn_3 = new GridColumn(grid, SWT.CENTER);
		gridColumn_3.setText("评标指标个数");
		gridColumn_3.setWidth(110);
		gridColumn_3.setMaxmumWidth(110);
		
		GridColumn gridColumn = new GridColumn(grid, SWT.CENTER);
		gridColumn.setText("是否已评审");
		gridColumn.setWidth(90);
		gridColumn.setMaxmumWidth(90);
		
		ESepator label = new ESepator(container, SWT.NONE);
		label.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		label.setBackground(ColorConstants.SEP_LIGHT_COLOR);
		
		Composite composite_1 = new Composite(container, SWT.NONE);
		composite_1.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		composite_1.setLayout(new GridLayout(3, false));
		composite_1.setBackgroundMode(SWT.INHERIT_FORCE);
		composite_1.setBackgroundImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/fill_bg.png"));
		
		tipsLabel = new Label(composite_1, SWT.NONE);
		tipsLabel.setFont(SWTResourceManager.getFont("微软雅黑", 9, SWT.NORMAL));
		tipsLabel.setLayoutData(new GridData(SWT.FILL, SWT.CENTER, true, false, 1, 1));
		tipsLabel.setForeground(ColorConstants.NOTE_TIPS_COLOR);
		tipsLabel.setText("");
		
		applyToBtn = new EButton(composite_1, SWT.NONE);
		applyToBtn.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				applyToAction();
			}
		});
		GridData gd_button = new GridData(SWT.FILL, SWT.CENTER, false, false, 1, 1);
		gd_button.widthHint = 90;
		applyToBtn.setLayoutData(gd_button);
		applyToBtn.setText(Messages.Button_confirm);
		applyToBtn.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/save.png"));
		
		EButton button = new EButton(composite_1, SWT.NONE);
		GridData gd_close_button = new GridData(SWT.LEFT, SWT.CENTER, false, false, 1, 1);
		gd_close_button.widthHint = 90;
		button.setLayoutData(gd_close_button);
		button.setText(Messages.Button_close);
		button.setImage(ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/btn/blue/close.png"));
		button.addMouseListener(new MouseAdapter() {
			public void mouseUp(MouseEvent e) {
				getShell().close();
			}
		});
		
		init();
		return container;
	}
	
	/**
	 * @Description: 初始化
	 * @author guoyr 
	 */
	private void init(){
		tenderProjectService = new TenderProjectService();
		tenderPackName = com.gx.obe.components.core.I18N.getString(I18NEnum.SUBPACKAGE, tenderProjectEntity.getTenderType());
		this.setTitle("批量复制评标步骤模型");
		tipsLabel.setText("将当前"+tenderPackName+"的评标步骤模型复制到选中的" + tenderPackName + "。");
		grid.setAutoHeight(true);
		grid.setAutoWidth(true);
		grid.setItemHeight(Constants.ROW_HIGHT);
		getTenderPackList();
	}


	/**
	 * @Description: 获得包件的评标信息
	 * @author guoyr 
	 */
	private void getTenderPackList(){
		List<Map<String, Object>> tenderPackList = tenderProjectService.getTenderPackEvaluationInfoList(tenderProjectEntity.getParentId(), tenderProjectEntity.getId());
		if(null != tenderPackList && tenderPackList.size() > 0){
			for(Map<String, Object> tenderPackMap : tenderPackList){
				GridItem gridItem = new GridItem(grid , SWT.NONE);
				String tenderId =  ObjectUtils.getObjString(tenderPackMap.get("TENDER_ID"));
				gridItem.setData("TENDER_ID", tenderId);
				gridItem.setText(0, ObjectUtils.getObjString(tenderPackMap.get("TENDER_NO")));
				gridItem.setText(1, ObjectUtils.getObjString(tenderPackMap.get("TENDER_NAME")));
				int factorCount = ObjectUtils.getIntValue(tenderPackMap.get("FACTOR_COUNT"), 0);
				int evaluationCount = ObjectUtils.getIntValue(tenderPackMap.get("EVALUATION_COUNT"), 0);
				gridItem.setText(2, factorCount +"");
				gridItem.setText(3, evaluationCount > 0 ? "已评审" : "未评审");
				gridItem.setChecked(0, true);
				if(factorCount == 0 || evaluationCount > 0){
					gridItem.setCheckVisible(0, false);
					gridItem.setChecked(0, false);
				}
				tenderPackItemMap.put(tenderId, gridItem);
			}
		}else {
			applyToBtn.setEnabled(false);
		}
	}
	
	/**
	 * @Description: 应用
	 * @author guoyr 
	 */
	private void applyToAction(){
		if(applyToBtn.getEnabled() && null != evaluationStepList){
			applyToBtn.setEnabled(false);
			final List<String> tenderIdList = GridUtils.getCheckedValues(grid, 0, "TENDER_ID", true);
			if(tenderIdList.size() > 0){
				final ProgressDialog progressDialog = new ProgressDialog(getShell(), "正在复制到其他" + tenderPackName+"...");
				new Thread(new Runnable(){
					public void run(){
						List<Future<TenderProjectEntity>> futureList = new ArrayList<Future<TenderProjectEntity>>();
						if(null != evaluationStepList && evaluationStepList.size() > 0){
							Document document = DocumentHelper.createDocument();
							Element rootElement = document.addElement("evaluationStepModel");
							for(EvaluationStep evaluationStep : evaluationStepList){
								rootElement.add(XmlBeanUtils.objectToElement(evaluationStep));
							}
							final File xmlFile = new File(FileConstants.TEMP_FOLDER.concat(tenderProjectEntity.getId()).concat("_evaluationStep.xml"));
							if(XmlUtils.createXmlFile(document, xmlFile.getAbsolutePath() , "UTF-8")){
								
								int maxThreadPoolCount = tenderIdList.size() > 5 ? 5 : tenderIdList.size();
								threadPoolExecutor = new ThreadPoolExecutor(maxThreadPoolCount, maxThreadPoolCount, 3, TimeUnit.SECONDS, new LinkedBlockingQueue<Runnable>(),new ThreadPoolExecutor.CallerRunsPolicy());
								for(String tenderId : tenderIdList){
									final TenderProjectEntity tenderPack = tenderProjectService.getById(tenderId);
									futureList.add(threadPoolExecutor.submit(new Callable<TenderProjectEntity>(){
										public TenderProjectEntity call() throws Exception {
											CreateEvaluationStepThread createEvaluationStepThread = new CreateEvaluationStepThread(tenderPack, xmlFile.getAbsolutePath(), new CallBack(){
												public void callBack(final boolean flag, final Object message) {
													setComplate(tenderPack.getId(), flag);
												}
											});
											createEvaluationStepThread.start();
											createEvaluationStepThread.join();
											return tenderPack;
										}
										
									}));
								}
								threadPoolExecutor.shutdown();
								
								for(Future<TenderProjectEntity> future : futureList){
									try {
										future.get();
									} catch (InterruptedException e) {
									} catch (ExecutionException e) {
									}
								}
								if(xmlFile.exists()){
									xmlFile.delete();
								}
							}
						}
						Display.getDefault().syncExec(new Runnable(){
							public void run(){
								if(null != progressDialog){
									progressDialog.close();
								}
								if(!applyToBtn.isDisposed()){
									applyToBtn.setEnabled(true);
								}
							}
						});
					}
				}).start();
				progressDialog.open();
			}else {
				MessageDialog.openShortMessage("请至少选择一个要应用的" + tenderPackName + "！");
				applyToBtn.setEnabled(true);
				return ;
			}
		}
		
	}
	
	private void setComplate(final String tenderId, final boolean flag){
		Display.getDefault().syncExec(new Runnable(){
			public void run(){
				GridItem gridItem = tenderPackItemMap.get(tenderId);
				if(null != gridItem && !gridItem.isDisposed()){
					if(flag){
						gridItem.setImage(0, ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/complate.png"));
						gridItem.setCheckVisible(0, false);
						gridItem.setChecked(0, false);
					}else {
						gridItem.setImage(0, ResourceManager.getPluginImage(IconsActivator.PLUGIN_ID, "icons/error_circle.png"));
					}
				}
			}
		});
	}
	@Override
	public boolean close() {
		if(null != threadPoolExecutor){
			threadPoolExecutor.shutdownNow();
		}
		return super.close();
	}

	/**
	 * Return the initial size of the dialog.
	 */
	@Override
	protected Point getInitialSize() {
		return new Point(664, 458);
	}
}
